@@ -1,5 +1,5 @@
 /*
- * $Id: libcitadel.c 7187 2009-03-04 18:54:51Z dothebart $
+ * $Id: libcitadel.c 8083 2009-11-24 23:26:33Z dothebart $
  *
  */
 
@@ -16,17 +16,23 @@
 #include "xdgmime/xdgmime.h"
 #include "libcitadellocal.h"
 
+extern int BaseStrBufSize;
 char *libcitadel_version_string(void) {
-	return "$Id: libcitadel.c 7187 2009-03-04 18:54:51Z dothebart $";
+	return "$Id: libcitadel.c 8083 2009-11-24 23:26:33Z dothebart $";
 }
 
 int libcitadel_version_number(void) {
 	return LIBCITADEL_VERSION_NUMBER;
 }
 
+void StartLibCitadel(size_t basesize)
+{
+	BaseStrBufSize = basesize;
+}
+
 void ShutDownLibCitadel(void)
 {
 	ShutDownLibCitadelMime();
-
+	WildFireShutdown();
 	xdg_mime_shutdown();
 }
