@@ -238,7 +238,7 @@ static int IncreaseBuf(StrBuf *Buf, int KeepOriginal, int DestSize)
  */
 void ReAdjustEmptyBuf(StrBuf *Buf, long ThreshHold, long NewSize)
 {
-	if (Buf->BufUsed > ThreshHold) {
+	if ((Buf != NULL) && (Buf->BufUsed > ThreshHold)) {
 		free(Buf->buf);
 		Buf->buf = (char*) malloc(NewSize);
 		Buf->BufUsed = 0;
@@ -255,6 +255,8 @@ void ReAdjustEmptyBuf(StrBuf *Buf, long ThreshHold, long NewSize)
  */
 long StrBufShrinkToFit(StrBuf *Buf, int Force)
 {
+	if (Buf == NULL)
+		return -1;
 	if (Force || 
 	    (Buf->BufUsed + (Buf->BufUsed / 3) > Buf->BufSize))
 	{
@@ -2070,7 +2072,8 @@ int StrBufTCP_read_buffered_line_fast(StrBuf *Line,
 {
 	const char *pche = NULL;
 	const char *pos = NULL;
-	int len, rlen;
+	const char *pLF;
+	int len, rlen, retlen;
 	int nSuccessLess = 0;
 	fd_set rfds;
 	const char *pch = NULL;
@@ -2078,6 +2081,7 @@ int StrBufTCP_read_buffered_line_fast(StrBuf *Line,
 	int IsNonBlock;
 	struct timeval tv;
 	
+	retlen = 0;
 	if ((Line == NULL) ||
 	    (Pos == NULL) ||
 	    (IOBuf == NULL) ||
@@ -2089,49 +2093,79 @@ int StrBufTCP_read_buffered_line_fast(StrBuf *Line,
 		return -1;
 	}
 
+	FlushStrBuf(Line);
+
 	pos = *Pos;
 	if ((IOBuf->BufUsed > 0) && 
 	    (pos != NULL) && 
 	    (pos < IOBuf->buf + IOBuf->BufUsed)) 
 	{
+		char *pcht;
+
 		pche = IOBuf->buf + IOBuf->BufUsed;
 		pch = pos;
+		pcht = Line->buf;
+
 		while ((pch < pche) && (*pch != '\n'))
-			pch ++;
+		{
+			if (Line->BufUsed + 10 > Line->BufSize)
+			{
+				long apos;
+				apos = pcht - Line->buf;
+				*pcht = '\0';
+				IncreaseBuf(Line, 1, -1);
+				pcht = Line->buf + apos;
+			}
+			*pcht++ = *pch++;
+			Line->BufUsed++;
+			retlen++;
+		}
+
+		len = pch - pos;
+		if (len > 0 && (*(pch - 1) == '\r') )
+		{
+			retlen--;
+			len --;
+			pcht --;
+			Line->BufUsed --;
+		}
+		*pcht = '\0';
+
 		if ((pch >= pche) || (*pch == '\0'))
+		{
+			FlushStrBuf(IOBuf);
+			*Pos = NULL;
 			pch = NULL;
+			pos = 0;
+		}
+
 		if ((pch != NULL) && 
 		    (pch <= pche)) 
 		{
-			rlen = 0;
-			len = pch - pos;
-			if (len > 0 && (*(pch - 1) == '\r') )
-				rlen ++;
-			StrBufSub(Line, IOBuf, (pos - IOBuf->buf), len - rlen);
-			*Pos = pch + 1;
-			return len - rlen;
+			if (pch + 1 >= pche) {
+				*Pos = NULL;
+				FlushStrBuf(IOBuf);
+			}
+			else
+				*Pos = pch + 1;
+			
+			return retlen;
 		}
-	}
-	
-	if (pos != NULL) {
-		if (pos > pche)
-			FlushStrBuf(IOBuf);
 		else 
-			StrBufCutLeft(IOBuf, (pos - IOBuf->buf));
-		*Pos = NULL;
+			FlushStrBuf(IOBuf);
 	}
+
+	/* If we come here, Pos is Unset since we read everything into Line, and now go for more. */
 	
-	if (IOBuf->BufSize - IOBuf->BufUsed < 10) {
+	if (IOBuf->BufSize - IOBuf->BufUsed < 10)
 		IncreaseBuf(IOBuf, 1, -1);
-		*Pos = NULL;
-	}
 
 	fdflags = fcntl(*fd, F_GETFL);
 	IsNonBlock = (fdflags & O_NONBLOCK) == O_NONBLOCK;
 
-	pch = NULL;
+	pLF = NULL;
 	while ((nSuccessLess < timeout) && 
-	       (pch == NULL) &&
+	       (pLF == NULL) &&
 	       (*fd != -1)) {
 		if (IsNonBlock)
 		{
@@ -2164,31 +2198,43 @@ int StrBufTCP_read_buffered_line_fast(StrBuf *Line,
 		}
 		else if (rlen > 0) {
 			nSuccessLess = 0;
+			pLF = IOBuf->buf + IOBuf->BufUsed;
 			IOBuf->BufUsed += rlen;
 			IOBuf->buf[IOBuf->BufUsed] = '\0';
-			if (IOBuf->BufUsed + 10 > IOBuf->BufSize) {
-				IncreaseBuf(IOBuf, 1, -1);
-				*Pos = NULL;
-			}
 			
 			pche = IOBuf->buf + IOBuf->BufUsed;
-			pch = IOBuf->buf;
-			while ((pch < pche) && (*pch != '\n'))
-				pch ++;
-			if ((pch >= pche) || (*pch == '\0'))
-				pch = NULL;
+			
+			while ((pLF < pche) && (*pLF != '\n'))
+				pLF ++;
+			if ((pLF >= pche) || (*pLF == '\0'))
+				pLF = NULL;
+
+			if (IOBuf->BufUsed + 10 > IOBuf->BufSize)
+			{
+				long apos;
+
+				if (pLF != NULL) apos = pLF - IOBuf->buf;
+				IncreaseBuf(IOBuf, 1, -1);	
+				if (pLF != NULL) pLF = IOBuf->buf + apos;
+			}
+
 			continue;
 		}
 	}
-	if (pch != NULL) {
+	*Pos = NULL;
+	if (pLF != NULL) {
 		pos = IOBuf->buf;
-		rlen = 0;
-		len = pch - pos;
-		if (len > 0 && (*(pch - 1) == '\r') )
-			rlen ++;
-		StrBufSub(Line, IOBuf, 0, len - rlen);
-		*Pos = pos + len + 1;
-		return len - rlen;
+		len = pLF - pos;
+		if (len > 0 && (*(pLF - 1) == '\r') )
+			len --;
+		StrBufAppendBufPlain(Line, ChrPtr(IOBuf), len, 0);
+		if (pLF + 1 >= IOBuf->buf + IOBuf->BufUsed)
+		{
+			FlushStrBuf(IOBuf);
+		}
+		else 
+			*Pos = pLF + 1;
+		return retlen + len;
 	}
 	*Error = ErrRBLF_NotEnoughSentFromServer;
 	return -1;
@@ -2860,6 +2906,9 @@ long StrBufUnescape(StrBuf *Buf, int StripBlanks)
 	char hex[3];
 	long len;
 
+	if (Buf == NULL)
+		return -1;
+
 	while ((Buf->BufUsed > 0) && (isspace(Buf->buf[Buf->BufUsed - 1]))){
 		Buf->buf[Buf->BufUsed - 1] = '\0';
 		Buf->BufUsed --;
